<?php

/**
 * @file
 * Contains \Drupal\publish_away\Plugin\Field\FieldType\PublishAwayTwitter.
 */

namespace Drupal\publish_away\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\PrepareCacheInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'publish_away' field type.
 *
 * @FieldType(
 *   id = "publish_away_twitter",
 *   label = @Translation("Publish away (Twitter)"),
 *   description = @Translation("This field stores a Twitter message in the database as a string."),
 *   default_widget = "publish_away_twitter_widget",
 *   default_formatter = "publish_away_default_formatter",
 * )
 */
// class PublishAwayTwitter extends ConfigFieldItemBase {
class PublishAwayTwitter extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'rows' => 4,
      'grouptitle' => 'Publish away on Twitter',
      'max_length' => 115,
      'placeholder' => '',
      'progress_indicator' => 'throbber',
      'preview_image_style' => 'thumbnail',
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return array(
      'columns' => array(
        'enabled' => array(
          'type' => 'int',
          'default' => 0,  
          'not null' => FALSE,
        ),
        'message' => array(
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE,
        ),
        'file_fid' => array(
          'type' => 'int',
          'not null' => FALSE,
        ),
      ),
    );
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, array &$form_state, $has_data) {
    $element['grouptitle'] = array(
      '#type' => 'textfield',
      '#title' => t('Title of fieldset'),
      '#default_value' => $this->getSetting('grouptitle'),
      '#disabled' => $has_data,  
    );
    $element['max_length'] = array(
      '#type' => 'number',
      '#title' => t('Max length of textarea'),
      '#default_value' => $this->getSetting('max_length'),
      '#required' => TRUE,
      '#disabled' => TRUE,
      '#min' => 1,  
    );
    // TextareaWidget.
    $element['rows'] = array(
      '#type' => 'number',
      '#title' => t('Rows'),
      '#default_value' => $this->getSetting('rows'),
      '#required' => TRUE,
      '#disabled' => TRUE,
      '#min' => 1,  
    );
    $element['placeholder'] = array(
      '#type' => 'textfield',
      '#title' => t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    );
    // FileWidget.
    $element['progress_indicator'] = array(
      '#type' => 'radios',
      '#title' => t('Progress indicator'),
      '#options' => array(
        'throbber' => t('Throbber'),
        'bar' => t('Bar with progress meter'),
      ),
      '#default_value' => $this->getSetting('progress_indicator'),
      '#description' => t('The throbber display does not show the status of uploads but takes up less space. The progress bar is helpful for monitoring progress on large uploads.'),
      '#weight' => 16,
      '#access' => file_progress_implementation(),
      '#disabled' => $has_data,  
    );
    // ImageWidget.
    $element['preview_image_style'] = array(
      '#title' => t('Preview image style'),
      '#type' => 'select',
      '#options' => image_style_options(FALSE),
      '#empty_option' => '<' . t('no preview') . '>',
      '#default_value' => $this->getSetting('preview_image_style'),
      '#description' => t('The preview image will be shown while editing the content.'),
      '#weight' => 15,
      '#disabled' => $has_data,  
    );
    return $element;
  }
  
  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['enabled'] = DataDefinition::create('boolean')
      ->setLabel(t('Enable sending'));
    $properties['message'] = DataDefinition::create('string')
      ->setLabel(t('Message'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $enabled = $this->get('enabled')->getValue();
    $message = $this->get('message')->getValue();
    return (int) $enabled && (trim($message) === '' || $message === NULL);
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();
    $constraints = parent::getConstraints();

    $max_length = $this->getSetting('max_length');
    $constraints[] = $constraint_manager->create('ComplexData', array(
      'message' => array(
        'Length' => array(
          'max' => $max_length,
          'maxMessage' => t('%name: may not be longer than @max characters.', array('%name' => $this->getFieldDefinition()->getLabel(), '@max' => $max_length)),
        )
      ),
    ));

    return $constraints;
  }
  
  /**
   * {@inheritdoc}
   */
  public function instanceSettingsForm(array $form, array &$form_state) {
    $element = array();

    return $element;
  }

}
